///////////////////////////////////////////////////////////////////////////
//                                                                       //
// NOTICE OF COPYRIGHT                                                   //
//                                                                       //
// Copyright (C) 2012    Andrés Mateus Vargas  <amvargash@gmail.com>     //            
//                                                                       //
// This program is free software; you can redistribute it and/or modify  //
// it under the terms of the GNU General Public License as published by  //
// the Free Software Foundation; either version 2 of the License, or     //
// (at your option) any later version.                                   //
//                                                                       //
// This program is distributed in the hope that it will be useful,       //
// but WITHOUT ANY WARRANTY; without even the implied warranty of        //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         //
// GNU General Public License for more details:                          //
//                                                                       //
//          http://www.gnu.org/copyleft/gpl.html                         //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include <TGClient.h>
#include <RQ_OBJECT.h>
#include <TMath.h>
#include <TVector.h>
#include <TGFrame.h>
#include <TGButton.h>
#include <TGLabel.h>
#include <TGNumberEntry.h>
#include <TLine.h>
#include <TArrow.h>
#include <TCanvas.h>
#include <TGaxis.h>

class WVector {
  RQ_OBJECT("WVector")
  private:
  TRootEmbeddedCanvas *fECanvas;
  TVector2 R1, R2, R3;
  TGNumberEntry *NEX1, *NEY1, *NEX2, *NEY2, *NEPhiR1, *NEPhiR2;
  TArrow *A1, *A2, *A3;
  TLine *L1R1, *L2R1, *L1R2, *L2R2, *L1R3, *L2R3;
  // TGColorSelect *CSR1, *CSR2;
  void ReDraw();
  public:
  WVector();
  ~WVector();
  void TBAddClicked();
  void TBSusClicked();

};

WVector::WVector() {
  
  TGMainFrame *fWVector;
  TGHorizontalFrame *HFrame;
  TGVerticalFrame *VFrame;
  TGGroupFrame *GFR1, *GFR2;
  TGLabel *LEX1, *LEX2, *LEY1, *LEY2, *LPhiR1, *LPhiR2;
  TGTextButton *TBAdd, *TBSus;

  LayoutA = new TGLayoutHints(kLHintsCenterX,2,2,2,2);
  LayoutB = new TGLayoutHints(kLHintsCenterX|kLHintsCenterY,5,5,5,5);
  fWVector = new TGMainFrame(gClient->GetRoot());
  fWVector->SetWindowName("Adici\363n y Sustracci\363n de Vectores");
  fWVector->SetCleanup(kDeepCleanup);
  

  HFrame = new TGHorizontalFrame(fWVector);
  VFrame = new TGVerticalFrame(HFrame);

  fECanvas = new TRootEmbeddedCanvas("ECanvas",HFrame,500,500);
  fECanvas->GetCanvas()->Range(-10,-10,10,10);


  EX = new TGaxis(-10,0,10,0,-10,10);
  EX->Draw();
  EY = new TGaxis(0,-10,0,10,-10,10);
  EY->Draw();

  /******EJEMPLO*****/
  R1 = TVector2(1,5);
  L1R1 = new TLine(R1.X(),0,R1.X(),R1.Y());
  L1R1->SetLineStyle(kDotted);
  L1R1->Draw();
  L2R1 = new TLine(0,R1.Y(),R1.X(),R1.Y());
  L2R1->SetLineStyle(kDotted);
  L2R1->Draw();

  R2 = TVector2(-4,-3);
  L1R2 = new TLine(R2.X(),0,R2.X(),R2.Y());
  L1R2->SetLineStyle(kDotted);
  L1R2->Draw();
  L2R2 = new TLine(0,R2.Y(),R2.X(),R2.Y());
  L2R2->SetLineStyle(kDotted);
  L2R2->Draw();

  R3 = R1+R2;
  L1R3 = new TLine(R3.X(),0,R3.X(),R3.Y());
  L1R3->SetLineStyle(kDotted);
  L1R3->Draw();
  L2R3 = new TLine(0,R3.Y(),R3.X(),R3.Y());
  L2R3->SetLineStyle(kDotted);
  L2R3->Draw();

  A1 = new TArrow(0,0,R1.X(),R1.Y());
  A1->SetLineColor(1);
  A2 = new TArrow(0,0,R2.X(),R2.Y());
  A2->SetLineColor(2);
  A3 = new TArrow(0,0,R3.X(),R3.Y());
  A3->SetLineColor(3);

  A1->Draw();
  A2->Draw();
  A3->Draw();
  
  fECanvas->GetCanvas()->SetEditable(kFALSE);
  /*****FIN EJEMPLO****/

  GFR1 = new TGGroupFrame(VFrame,"Vector R1");
  LEX1 = new TGLabel(GFR1,"X1 : ");
  NEX1 = new TGNumberEntry(GFR1);
  NEX1->SetNumStyle(TGNumberFormat::kNESRealTwo);
  NEX1->SetNumber(R1.X());
  LEY1 = new TGLabel(GFR1,"Y1 : ");
  NEY1 = new TGNumberEntry(GFR1);
  NEY1->SetNumStyle(TGNumberFormat::kNESRealTwo);
  NEY1->SetNumber(R1.Y());
  LPhiR1 = new TGLabel(GFR1,"Phi : ");
  NEPhiR1 = new TGNumberEntry(GFR1);
  NEPhiR1->SetNumStyle(TGNumberFormat::kNESRealTwo);
  // NEPhiR1->SetNumber(R1.Phi());
  // CSR1 = new TGColorSelect(GFR1);
  // CSR1->SetColor(A1->GetLineColor());
  GFR1->AddFrame(LEX1);
  GFR1->AddFrame(NEX1);
  GFR1->AddFrame(LEY1);
  GFR1->AddFrame(NEY1);
  GFR1->AddFrame(LPhiR1);
  GFR1->AddFrame(NEPhiR1);
  // GFR1->AddFrame(CSR1);

  GFR2 = new TGGroupFrame(VFrame,"Vector R2");
  LEX2 = new TGLabel(GFR2,"X2 : ");
  NEX2 = new TGNumberEntry(GFR2);
  NEX2->SetNumStyle(TGNumberFormat::kNESRealTwo);
  NEX2->SetNumber(R2.X());
  LEY2 = new TGLabel(GFR2,"Y2 : ");
  NEY2 = new TGNumberEntry(GFR2);
  NEY2->SetNumStyle(TGNumberFormat::kNESRealTwo);
  NEY2->SetNumber(R2.Y());
  LPhiR2 = new TGLabel(GFR2,"Phi : ");
  NEPhiR2 = new TGNumberEntry(GFR2);
  NEPhiR2->SetNumStyle(TGNumberFormat::kNESRealTwo);
  // NEPhiR2->SetNumber(R2.Phi());
  // CSR2 = new TGColorSelect(GFR2);
  // CSR2->SetColor(A2->GetLineColor());
  GFR2->AddFrame(LEX2);
  GFR2->AddFrame(NEX2);
  GFR2->AddFrame(LEY2);
  GFR2->AddFrame(NEY2);
  GFR2->AddFrame(LPhiR2);
  GFR2->AddFrame(NEPhiR2);
  // GFR2->AddFrame(CSR2);

  TBAdd = new TGTextButton(VFrame,"R1 + R2");
  TBAdd->Connect("Clicked()","WVector",this,"TBAddClicked()");
  TBSus = new TGTextButton(VFrame,"R1 - R2");
  TBSus->Connect("Clicked()","WVector",this,"TBSusClicked()");

  VFrame->AddFrame(GFR1,LayoutA);
  VFrame->AddFrame(GFR2,LayoutA);
  VFrame->AddFrame(TBAdd,LayoutA);
  VFrame->AddFrame(TBSus,LayoutA);

  HFrame->AddFrame(fECanvas,LayoutA);
  HFrame->AddFrame(VFrame,LayoutB);
  
  fWVector->AddFrame(HFrame,LayoutB);
  fWVector->Resize(fWVector->GetDefaultSize());
  fWVector->MapSubwindows();
  fWVector->MapWindow();

}

void WVector::ReDraw(){

  fECanvas->GetCanvas()->SetEditable(kTRUE);

  A1->SetX2(R1.X());
  A1->SetY2(R1.Y());
  A1->Draw();
  L1R1->SetX1(R1.X());
  L1R1->SetY1(0);
  L1R1->SetX2(R1.X());
  L1R1->SetY2(R1.Y());
  L1R1->Draw();
  L2R1->SetX1(0);
  L2R1->SetY1(R1.Y());
  L2R1->SetX2(R1.X());
  L2R1->SetY2(R1.Y());
  L2R1->Draw();

  A2->SetX2(R2.X());
  A2->SetY2(R2.Y());
  A2->Draw();
  L1R2->SetX1(R2.X());
  L1R2->SetY1(0);
  L1R2->SetX2(R2.X());
  L1R2->SetY2(R2.Y());
  L1R2->Draw();
  L2R2->SetX1(0);
  L2R2->SetY1(R2.Y());
  L2R2->SetX2(R2.X());
  L2R2->SetY2(R2.Y());
  L2R2->Draw();

  A3->SetX2(R3.X());
  A3->SetY2(R3.Y());
  A3->Draw();
  L1R3->SetX1(R3.X());
  L1R3->SetY1(0);
  L1R3->SetX2(R3.X());
  L1R3->SetY2(R3.Y());
  L1R3->Draw();
  L2R3->SetX1(0);
  L2R3->SetY1(R3.Y());
  L2R3->SetX2(R3.X());
  L2R3->SetY2(R3.Y());
  L2R3->Draw();
  
  gPad->Update();
  
  fECanvas->GetCanvas()->SetEditable(kFALSE);
}

void WVector::TBAddClicked(){

  R1.Set(NEX1->GetNumber(),NEY1->GetNumber());
  R1 = R1.Rotate(NEPhiR1->GetNumber()*(TMath::Pi()/180));
  R2.Set(NEX2->GetNumber(),NEY2->GetNumber());
  R2 = R2.Rotate(NEPhiR2->GetNumber()*(TMath::Pi()/180));
  R3 = R1 + R2;

  ReDraw();

 
}

void WVector::TBSusClicked(){

  R1.Set(NEX1->GetNumber(),NEY1->GetNumber());
  R1 = R1.Rotate(NEPhiR1->GetNumber()*(TMath::Pi()/180));
  R2.Set(NEX2->GetNumber(),NEY2->GetNumber());
  R2 = R2.Rotate(NEPhiR2->GetNumber()*(TMath::Pi()/180));

  R3 = R1 - R2;
  
  ReDraw();

}

void Vector() {

  App = new WVector();
 
}
